@@ -1,6 +1,6 @@
 !--
 !----------------------------------------------------------------------
-!     Copyright (c) 2002-2008 Shin-ichi Takehiro. All rights reserved.
+! Copyright (c) 2002-2010 SPMODEL Development Group. All rights reserved.
 !----------------------------------------------------------------------
 !ɽ  wt_mpi_module
 !
@@ -59,7 +59,9 @@
 !      2008/06/11  ݹ  OPENMP åƳ
 !      2008/06/11  ݹ  źѹ
 !      2009/02/27  ʿ RDoc ѤΥȤ
-!
+!      2010/01/07  ʿ  RDoc ѤΥɥȽ, 
+!                              include 'mpif.h' -> use mpi
+
 !
 !      ǡ index
 !        x :          y :     v : (ʬʻ)     z : ư
@@ -84,7 +86,7 @@ module wt_mpi_module
   != wt_mpi_module
   !
   ! Authors:: Shin-ichi Takehiro, Youhei SASAKI
-  ! Version:: $Id: wt_mpi_module.f90,v 1.8 2009-02-27 05:21:52 uwabami Exp $
+  ! Version:: $Id: wt_mpi_module.f90,v 1.9 2010-02-18 15:28:27 uwabami Exp $
   ! Copyright&License:: See COPYRIGHT[link:../../COPYRIGHT]
   !
   !== 
@@ -374,6 +376,7 @@ module wt_mpi_module
   ! 
   use dc_message
   use lumatrix
+  use mpi
   use w_base_mpi_module
   use w_deriv_mpi_module
   use w_integral_mpi_module
@@ -382,7 +385,6 @@ module wt_mpi_module
   use wt_module
 
   implicit none
-  include 'mpif.h'
   integer :: ierr
 
   private
