@@ -1,6 +1,6 @@
 !--
 !----------------------------------------------------------------------
-!     Copyright (c) 2008 Shin-ichi Takehiro. All rights reserved.
+! Copyright(c) 2008-2010 SPMDODEL Development Group. All rights reserved.
 !----------------------------------------------------------------------
 !ɽ  wq_module
 !
@@ -27,6 +27,11 @@
 !      2008/07/21  ݹ  ׻ѹ
 !      2009/01/09  ݹ  wu_Initial åդɲ
 !      2009/01/29  ʿ Ȥ RDoc Ѥ
+!      2009/07/31  ݹ  ׻ threadprivate (OpenMP)
+!      2009/12/06  ݹ  ƬȤ
+!      2010/03/10  ʿ  threadprivate (ѥ¸)
+!      2010/04/17  ݹ  wr_DivRad_xyr ɲ, wr_Div_xyr_xyr_xyr ѹ
+!                            wq_LaplaPol2Pol_wq η׻ˡѹ
 !
 !
 !      ǡ index
@@ -51,7 +56,7 @@ module wq_module
   != wq_module
   !
   ! Authors:: Shin-ichi Takehiro, Youhei SASAKI
-  ! Version:: $Id: wq_module.f90,v 1.8 2009-02-28 21:33:48 uwabami Exp $
+  ! Version:: $Id: wq_module.f90,v 1.13 2010-04-15 23:56:55 takepiro Exp $
   ! Copyright&License:: See COPYRIGHT[link:../COPYRIGHT]
   !
   !== 
@@ -102,7 +107,7 @@ module wq_module
   !   * ʿڥȥǡγǼΤϴؿ l_nm, nm_l ˤäĴ٤
   !     ȤǤ. 
   !   * ư¥ڥȥǡγǼˡˤĤƤ aq_module.f90 
-  !     ȤΤ. 
+  !     ȤΤ. km < 2*im ǤʤФʤʤ. 
   !
   ! * nmr : ʿڥȥǡ¤ 3 .
   !   * ѿμȼ real(8), dimension(0:nm,-nm:nm,km). 
@@ -163,6 +168,8 @@ module wq_module
   ! wq_RadDRad_wq       :: ڥȥǡưʬ r/r Ѥ
   ! wr_DivRad_wq        :: ڥȥǡȯưʬ
   !                        1/r^2 /r r^2 = /r + 2/r Ѥ
+  ! wr_DivRad_xyr       :: ʻǡȯưʬ
+  !                        1/r^2 /r r^2 = /r + 2/r Ѥ
   ! wr_RotDRad_wq       :: ڥȥǡ˲žưʬ
   !                        1/r /rr = /r + 1/r Ѥ
   ! wr_RotDRad_wr       :: ڥȥǡ˲žưʬ
@@ -198,7 +205,7 @@ module wq_module
   !                      Ѥ
   ! wq_QOperator_wq   :: ڥȥǡ˱黻
   !                      Q=(k-1/2(L2 k+ kL2)) Ѥ
-  ! wq_RadRot_xyr_xyr :: ٥ȥ v α٤ư¥٥ȥ r  r(v) 
+  ! wr_RadRot_xyr_xyr :: ٥ȥ v α٤ư¥٥ȥ r  r(v) 
   !                      ׻
   ! wr_RadRotRot_xyr_xyr_xyr :: ٥ȥ v  r(ߢv) ׻
   ! wq_RadRotRot_xyr_xyr_xyr :: ٥ȥ v  r(ߢv) ׻
@@ -216,21 +223,11 @@ module wq_module
   !
   !==== 
   !
-  ! wq_BoundariesTau, wr_BoundariesGrid, wq_Boundaries                   ::
-  !     ǥꥯ, Υޥ󶭳ŬѤ(ˡ, ˡ)
-  ! wq_TorBoundariesTau, wr_TorBoundariesGrid, wq_TorBoundaries          ::
-  !     ®٥ȥݥƥ󥷥ζŬѤ(ˡ,ˡ)
-  ! wq_LaplaPol2Pol_wq                                                   :: 
-  !     ®٥ݥݥƥ󥷥릵^2 
-  ! wq_TorMagBoundariesTau, wr_TorMagBoundariesGrid, wq_TorMagBoundaries ::
-  !     ȥݥƥ󥷥ζŬѤ(ˡ, ˡ)
-  ! wq_PolMagBoundariesTau, wr_PolMagBoundariesGrid, wq_PolMagBoundaries ::
-  !     ȥݥƥ󥷥붭ζŬѤ(ˡ, ˡ)
   ! wq_BoundaryTau, wr_BoundaryGrid, wq_Boundary                         ::
   !     ǥꥯ, Υޥ󶭳ŬѤ(ˡ, ˡ)
   ! wq_TorBoundaryTau, wr_TorBoundaryGrid, wq_TorBoundary                ::
   !     ®٥ȥݥƥ󥷥ζŬѤ(ˡ,ˡ)       
-  ! wr_LaplaPol2Pol_wr, wq_LaplaPol2Pol_wq                               ::
+  ! wq_LaplaPol2Pol_wq, wq_LaplaPol2PolTau_wq                            ::
   !     ®٥ݥݥƥ󥷥릵^2
   !     (Ϥ줾ʻӥڥȥ뷸)
   ! wq_TorMagBoundaryTau, wr_TorMagBoundaryGrid, wq_TorMagBoundary       ::
@@ -240,33 +237,19 @@ module wq_module
   !
   !==== ʬʿ(3 ǡ)
   !
-  ! IntLonLatRad_xyz, AvrLonLatRad_xyz :: 3 ʻǡ
-  !                                       ΰʬʿ
-  ! z_IntLonLat_xyz, z_AvrLonLat_xyz   :: 3 ʻǡ
-  !                                       ٷ(ʿ)ʬʿ
-  ! y_IntLonRad_xyz, y_AvrLonRad_xyz   :: 3 ʻǡ
-  !                                       ưʬʿ
-  ! z_IntLatRad_xyz, z_AvrLatRad_xyz   :: 3 ʻǡ
-  !                                       ư(Ҹ)ʬʿ
-  ! yz_IntLon_xyz, yz_AvrLon_xyz       :: 3 ʻǡ
-  !                                       ʬʿ
-  ! xz_IntLat_xyz, xz_AvrLat_xyz       :: 3 ʻǡ
-  !                                       ʬʿ
-  ! xz_IntRad_xyz, xz_AvrRad_xyz       :: 3 ʻǡ
-  !                                       ưʬʿ
   ! IntLonLatRad_xyr, AvrLonLatRad_xyr :: 3 ʻǡ
   !                                       ΰʬʿ
-  ! z_IntLonLat_xyr, z_AvrLonLat_xyr   :: 3 ʻǡ
+  ! r_IntLonLat_xyr, r_AvrLonLat_xyr   :: 3 ʻǡ
   !                                       ٷ(ʿ)ʬʿ
   ! y_IntLonRad_xyr, y_AvrLonRad_xyr   :: 3 ʻǡ
   !                                       ưʬʿ
-  ! z_IntLatRad_xyr, z_AvrLatRad_xyr   :: 3 ʻǡ
+  ! r_IntLatRad_xyr, r_AvrLatRad_xyr   :: 3 ʻǡ
   !                                       ư(Ҹ)ʬʿ
-  ! yz_IntLon_xyr, yz_AvrLon_xyr       :: 3 ʻǡ
+  ! yr_IntLon_xyr, yr_AvrLon_xyr       :: 3 ʻǡ
   !                                       ʬʿ
-  ! xz_IntLat_xyr, xz_AvrLat_xyr       :: 3 ʻǡ
+  ! xr_IntLat_xyr, xr_AvrLat_xyr       :: 3 ʻǡ
   !                                       ʬʿ
-  ! xz_IntRad_xyr, xz_AvrRad_xyr       :: 3 ʻǡ
+  ! xr_IntRad_xyr, xr_AvrRad_xyr       :: 3 ʻǡ
   !                                       ưʬʿ
   !
   !==== ʬʿ(2 ǡ)
@@ -325,7 +308,7 @@ module wq_module
   public wr_DivRad_wq, wr_RotDRad_wq, wr_RotDRad_wr, wq_Lapla_wq
   public wq_RotDRad_wr
   public xyr_GradLon_wq, xyr_GradLat_wq
-  public wr_DivLon_xyr, wr_DivLat_xyr
+  public wr_DivLon_xyr, wr_DivLat_xyr, wr_DivRad_xyr
   public wr_Div_xyr_xyr_xyr, xyr_Div_xyr_xyr_xyr
   public xyr_RotLon_wq_wq, xyr_RotLat_wq_wq, wr_RotRad_xyr_xyr
 
@@ -472,7 +455,7 @@ module wq_module
 
      wr_Rad = spread(r_Rad,1,(nm+1)*(nm+1))
 
-    call MessageNotify('M','wu_initial','wu_module (2009/01/09) is initialized')
+    call MessageNotify('M','wq_initial','wq_module (2010/4/17) is initialized')
 
    end subroutine wq_initial
 
@@ -583,6 +566,28 @@ module wq_module
 
     end function wr_DivRad_wq
 
+    function wr_DivRad_xyr(xyr)
+      ! 
+      ! ʻǡȯưʬ
+      !
+      !       1/r^2 /r (r^2  = /r + 2/= 1/r/(r.) + 1/r
+      !
+      ! Ѥ.
+      !
+      ! δؿϤư¼ȿʿȿζۤʤäƤ뤳Ȥ
+      ! ꤷƤ. פƤˤ wr_DigRad_wq Ѥ뤳.
+      !
+      real(8), dimension(0:im-1,jm,km), intent(in) :: xyr
+      !(in) 2 Ĵȡӥեڥȥǡ
+
+      real(8), dimension((nm+1)*(nm+1),km)             :: wr_DivRad_xyr
+      !(out) ȯưʬѤ줿ʿڥȥư³ʻǡ
+
+      wr_DivRad_xyr = wr_wq(wq_RadDRad_wq(wq_xyr(xyr_Rad*xyr)))/wr_Rad**2 &
+                    + 1/wr_Rad * wr_xyr(xyr)
+
+    end function wr_DivRad_xyr
+
     function wr_RotDRad_wq(wq)
       !
       ! ϥڥȥǡ˲žưʬ
@@ -742,6 +747,8 @@ module wq_module
       !
       ! ȷ׻.
       !
+      ! 3ʬư¼ȿʿȿζƤ뤳ȤꤷƤ.
+      !
       real(8), dimension(0:im-1,1:jm,km), intent(in) :: xyr_Vlon
       !(in) ٥ȥηʬ
 
@@ -756,7 +763,7 @@ module wq_module
 
       wr_Div_xyr_xyr_xyr =   wr_DivLon_xyr(xyr_Vlon) &
                            + wr_DivLat_xyr(xyr_Vlat) &
-                           + wr_DivRad_wq(wq_xyr(xyr_Vrad))
+                           + wr_DivRad_xyr(xyr_Vrad)
 
     end function wr_Div_xyr_xyr_xyr
 
@@ -2258,9 +2265,6 @@ module wq_module
       real(8), dimension(:,:,:), allocatable  :: alu     ! ΰ׻
       integer, dimension(:,:), allocatable    :: kp      ! ΰ׻
 
-      real(8), dimension(:,:,:), allocatable  :: alub    ! ׻
-      integer, dimension(:,:), allocatable    :: kpb     ! ׻
-
       real(8), dimension((nm+1)*(nm+1),km)    :: wr_work
       real(8), dimension((nm+1)*(nm+1),0:lm)  :: wq_work
       logical                                 :: rigid   ! 
@@ -2269,7 +2273,6 @@ module wq_module
       logical :: new_matrix = .false.
       integer :: l,n, lend
       save    :: alu, kp, first
-      save    :: alub, kpb
 
       if (.not. present(cond)) then
          rigid=.TRUE. 
@@ -2295,11 +2298,7 @@ module wq_module
 
          if ( allocated(alu) ) deallocate(alu)
          if ( allocated(kp) ) deallocate(kp)
-         if ( allocated(alub) ) deallocate(alub)
-         if ( allocated(kpb) ) deallocate(kpb)
          allocate(alu((nm+1)*(nm+1),0:lm,0:lm),kp((nm+1)*(nm+1),0:lm))
-         allocate(alub((nm+1)*(nm+1),0:lm,0:lm),kpb((nm+1)*(nm+1),0:lm))
-
 
          !---- ΰ׻ѹ -----
          do l=0,lm
@@ -2318,24 +2317,7 @@ module wq_module
             enddo
          enddo
 
-         ! alu(:,:,nd(n))  0 ʤΤ 1 򤤤Ƥ. 
-         ! l=nd(n) ʬ϶Ƿ. 
-         do n=1,(nm+1)**2
-            if ( mod(nd(n),2) .eq. mod(lm,2) ) then
-               alu(n,lm,nd(n)) = 1.0D0
-            else
-               alu(n,lm-1,nd(n)) = 1.0D0
-            endif
-         enddo
-
-         call ludecomp(alu,kp)
-
-         !---- ׻ѹ -----
-
-         alub = 0.0D0
-         do l=0,lm
-            alub(:,l,l) = 1.0D0
-         enddo
+         !---- ׻ -----
 
          do l=0,lm
             wq_work=0.0D0 ; wq_work(:,l)=1.0D0
@@ -2344,7 +2326,12 @@ module wq_module
             ! ưŪ. ή϶ǰ
             !     l=nd(n) ʬ򶭳Ƿ. 
             do n=1,(nm+1)**2
-               alub(n,nd(n),l) = wr_work(n,km)
+               if ( mod(nd(n),2) .eq. mod(lm,2) ) then
+                  lend = lm
+               else
+                  lend = lm-1
+               endif
+               alu(n,lend-2,l) = wr_work(n,km)
             enddo
 
             ! ϳŪǴ 
@@ -2363,29 +2350,11 @@ module wq_module
                else
                   lend = lm-1
                endif
-               alub(n,lend,l) = wr_work(n,km)
+               alu(n,lend,l) = wr_work(n,km)
             enddo
          enddo
 
-         ! طʤȤ 0 .
-         do n=1,(nm+1)**2
-            if ( mod(nd(n),2) .eq. mod(lm,2) ) then
-               lend = lm
-            else
-               lend = lm-1
-            endif
-
-            do l=0,nd(n)-1
-               alub(n,nd(n),l) = 0.0D0
-               alub(n,lend,l) = 0.0D0
-            enddo
-            do l=nd(n)+1,lm,2
-               alub(n,nd(n),l) = 0.0D0
-               alub(n,lend,l) = 0.0D0
-            enddo
-         enddo
-
-         call ludecomp(alub,kpb)
+         call ludecomp(alu,kp)
 
          if ( rigid ) then
             call MessageNotify('M','wq_LaplaPol2PolTau_wq',&
@@ -2399,19 +2368,18 @@ module wq_module
       ! ΰ׻
       wq_work = wq
 
-      wq_work = lusolve(alu,kp,wq_work)
-
       ! ׻
       do n=1,(nm+1)**2
-         wq_work(n,nd(n)) = 0
          if ( mod(nd(n),2) .eq. mod(lm,2) ) then
             wq_work(n,lm)   = 0
+            wq_work(n,lm-2)   = 0
          else
             wq_work(n,lm-1) = 0
+            wq_work(n,lm-3) = 0
          endif
       enddo
 
-      wq_laplapol2polTau_wq = lusolve(alub,kpb,wq_work)
+      wq_laplapol2polTau_wq = lusolve(alu,kp,wq_work)
 
     end function wq_LaplaPol2PolTau_wq
 
